@@ -126,7 +126,8 @@ class TopgreenerCloud:
                 logger.debug("Received response: %s", data)
 
                 if "success" not in data or data["success"] is False:
-                    raise TuyaException("Request unsuccessful! Params: " + data["errorCode"] + " " + data["errorMsg"])
+                    raise TuyaException("Request unsuccessful! Params: " + data["errorCode"] + " " + data["errorMsg"],
+                                        data["errorCode"])
 
                 return data["result"]
 
