@@ -1,6 +1,9 @@
+from datetime import datetime, timedelta
 from typing import Any, Optional, Dict, Union, List, TypeVar, Type, cast, Callable
 from enum import Enum
 
+from topgreenerapi.models import device_schema
+
 T = TypeVar("T")
 EnumT = TypeVar("EnumT", bound=Enum)
 
@@ -163,6 +166,7 @@ class TuyaDevice:
     dp_max_time: int
     product_id: str
     dps: Dict[str, Union[bool, int]]
+    assumed_dps: Dict[str, Union[bool, int, Any]]
     ip: str
     active_time: int
     category_code: str
@@ -200,6 +204,32 @@ class TuyaDevice:
         self.category = category
         self.local_key = local_key
 
+        self.assumed_dps = {}
+        for dp in dps:
+            self.assumed_dps[dp] = {"value": dps[dp], "expiry": None}
+
+    def setDp(self, ability: 'device_schema.DeviceSchema', value, assumed=False):
+        if assumed:
+            now = datetime.now()
+            exp = now + timedelta(seconds=5)
+            self.assumed_dps[str(ability.id)] = {"value": value, "expiry": exp}
+        else:
+            self.dps[str(ability.id)] = value
+            val = self.assumed_dps[str(ability.id)]
+            if "expiry" in val and not val["expiry"] is None:
+                if val["expiry"] < datetime.now():
+                    self.assumed_dps[str(ability.id)] = {"value": value, "expiry": None}
+
+    def getDp(self, ability: 'device_schema.DeviceSchema', assumed=False) -> Any:
+        if assumed:
+            value = self.assumed_dps[str(ability.id)]
+            if "value" in value:
+                return value["value"]
+            else:
+                return value
+        else:
+            return self.dps[str(ability.id)]
+
     @staticmethod
     def from_dict(obj: Any) -> 'TuyaDevice':
         assert isinstance(obj, dict)
@@ -272,6 +302,15 @@ class TuyaDevice:
         model.timezone_id = timezone_id
         model.local_key = local_key
 
+        for asp in model.assumed_dps:
+            val = model.assumed_dps[asp]
+            if "expiry" in val and not val["expiry"] is None:
+                if val["expiry"] < datetime.now():
+                    # assumption expired
+                    model.assumed_dps[asp] = {"value": dps[asp], "expiry": None}
+            else:
+                model.assumed_dps[asp] = {"value": dps[asp], "expiry": None}
+
         return model
 
     def to_dict(self) -> dict:
